/**
 * Copyright (C) 2010-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.atmos;

import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.CloudException;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.dc.DataCenterServices;
import org.dasein.cloud.jclouds.atmos.storage.AtmosStorageServices;
import org.jclouds.blobstore.BlobStoreContext;
import org.jclouds.blobstore.BlobStoreContextFactory;
import org.jclouds.concurrent.MoreExecutors;
import org.jclouds.concurrent.config.ExecutorServiceModule;
import org.jclouds.logging.log4j.config.Log4JLoggingModule;

import com.google.inject.Module;

public class Atmos extends AbstractCloud {
    static private final Logger logger = Logger.getLogger(Atmos.class);
    
    public Atmos() { }

    @Override
    public String getCloudName() {
        String cloud = getContext().getCloudName();
        
        if( cloud != null ) {
            return cloud;
        }
        return "Atmos Online";
    }

    public BlobStoreContext getAtmosContext() throws CloudException {
        logger.debug("enter - getAtmosContext()");
        try {
            ProviderContext ctx = getContext();
            String account = new String(ctx.getAccessPublic());
            String key = new String(ctx.getAccessPrivate());
            
            String userId = account + "/" + ctx.getAccountNumber();
                
            ArrayList<Module> modules = new ArrayList<Module>();
            
            modules.add(new Log4JLoggingModule());
            modules.add(new ExecutorServiceModule(MoreExecutors.sameThreadExecutor(), MoreExecutors.sameThreadExecutor()));
            return (new BlobStoreContextFactory()).createContext("synaptic-storage", userId, key, modules);
            //return AtmosStorageContextFactory.createContext(new URI(ctx.getEndpoint()), userId, key, 
        }
        finally {
            logger.debug("exit - getAtmosContext()");
        }
    }
    
    @Override
    public DataCenterServices getDataCenterServices() {
        CloudProvider compute = getComputeCloud();
        
        if( compute != null ) {
            return compute.getDataCenterServices();
        }
        return new DataCenters(this);
    }

    @Override
    public String getProviderName() {
        String provider = getContext().getProviderName();
        
        if( provider != null ) {
            return provider;
        }
        return "EMC";
    }
    
    @Override
    public AtmosStorageServices getStorageServices() {
        return new AtmosStorageServices(this);
    }
    
    @Override
    public String testContext() {
        try {
            if( !getStorageServices().getBlobStoreSupport().isSubscribed() ) {
                return null;
            }
            return getContext().getAccountNumber();
        }
        catch( Throwable t ) {
            logger.warn("Failed to test Atmos connection context: " + t.getMessage());
            if( logger.isDebugEnabled() ) {
                t.printStackTrace();
            }
            return null;
        }
    }
}
